#
# load mass library
library(MASS)
#
# ------------------------------------------------------------
# Metropolis hastings using multivariate normal proposal
# for logistic regression example
# 
# 13 Sep 2007 Ioannis Ntzoufras 
#             Athens University of Economics and Business
# ------------------------------------------------------------
#

ex2.3.mdrw<-function(y=wais$senility, x=wais$wais, prop.sd=0.3, iter=2500, beta0=c(0,0)){
#-------------------------------------
calculate.loglike<-function(b,X=x,Y=y){
 x<-X; y<-y
 n<-length(x); X<-cbind( rep(1,n), x )
 precision<-700
 eta<-b[1]+b[2]*x 
 logq <- log(1+exp(eta))
 logq[eta>precision]<-eta[eta>precision]
 loglike<- sum(  y*eta  - logq )
 eta[eta>precision]<-precision 
 h  <- 1/((1+exp(-eta))*(1+exp(eta)))
 H  <- t(X) %*% diag( h ) %*% X
 return( list(loglike=loglike, H=H) )
}
#---------------------------------------
n<-length(y); X<-cbind( rep(1,n), x ) 
Iterations<-iter
mu.beta<-c(0,0); s.beta<-c(100,100) 
c.beta<- prop.sd
beta <- matrix(nrow=Iterations, ncol=2) 
acc.prob <- 0 
current.beta<-beta0
for (t in 1:Iterations){  
 cur<-calculate.loglike( current.beta )
 cur.T<-(1/c.beta^2)*(cur$H+diag(1/s.beta^2))
 prop.beta<- mvrnorm( 1, current.beta, solve(cur.T)) 
 prop<-calculate.loglike( prop.beta )
 prop.T  <- (1/c.beta^2)* (prop$H+diag(1/s.beta^2))
 loga <-( prop$loglike-cur$loglike
 +sum(dnorm(prop.beta,   mu.beta,s.beta,log=TRUE)) 
 -sum(dnorm(current.beta,mu.beta,s.beta,log=TRUE)) 
 + as.numeric(0.5*log( det(prop.T) ) - 0.5 * t(current.beta - prop.beta) %*% prop.T %*% (current.beta - prop.beta))
 - as.numeric(0.5*log( det(cur.T ) ) - 0.5 * t(prop.beta - current.beta) %*% cur.T  %*% (prop.beta- current.beta )) )
 
 u<-runif(1)  
 u<-log(u)  
 if( u < loga ) { 
		current.beta<-prop.beta	  
		acc.prob <- acc.prob+1 
		}  
 beta[t,]<-current.beta  
 } 
 print(acc.prob/Iterations)
 return(beta)
}

beta2<-ex2.3.mdrw(prop.sd=2.5, iter=55000)

